/**
  Maxi2 : an inline Picture Magnifier for PmWiki
  Written by (c) Petko Yotov 2009-2021

  This text is written for PmWiki; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published
  by the Free Software Foundation; either version 3 of the License, or
  (at your option) any later version. See pmwiki.php for full details
  and lack of warranty.

  Copyright 2006-2021 Petko Yotov www.pmwiki.org/petko
  
  This is a major rewrite of the 12 years old PmWiki:Cookbook:Maxi 
  recipe to work with mouse gestures (drag-to-pan, scroll-to-zoom)
  and with touch gestures (pinch-to-zoom and drag-to-pan).
  
  Touch gestures via the Zoom.js library (MIT License) by Anita Vasu
  adapted and extended by Petko Yotov
*/

document.addEventListener('DOMContentLoaded', function(){

  function log() { for(var a of arguments) console.log(a); }

  
  var mousePreviewDClick = false;
  function previewMouseClick(e) {
    if(e.button !== 0) return;
    if(e.target.closest('.maxifullscreen')) return;
    e.preventDefault();
    var zid = parseInt(this.closest('.maxidiv').dataset.zoomid);
   
    if(e.type == "mouseup" || e.type == "mouseout" ) {
      MyZoom[zid].petko = false;
    }
    else if(e.type == "mousedown") {
      if(mousePreviewDClick) {
        MyZoom[zid].petko = false;
        mousePreviewDClick = false;
        resetMyZoom(zid);
      }
      else {
        transformElement(zid, -1);
        
        MyZoom[zid].petko = { touchX:e.clientX, touchY:e.clientY, 
            translateX:transformCache[zid].x, translateY:transformCache[zid].y};
            
        mousePreviewDClick = setTimeout(function(){
          mousePreviewDClick = false;
        }, 400);
      }
    }
  }

  function previewMouseMove(e) {
    var zid = parseInt(this.dataset.zoomid);
    
    if(! MyZoom[zid].petko) return; // mouse is up
    e.preventDefault();
    
    var moveX = e.clientX, moveY = e.clientY, 
      touchX = MyZoom[zid].petko.touchX, touchY = MyZoom[zid].petko.touchY,
      translateX = MyZoom[zid].petko.translateX, translateY = MyZoom[zid].petko.translateY;
    
    var deltaX = moveX - touchX;
    var newX = translateX + deltaX;
    
    var deltaY = moveY - touchY;
    var newY = translateY + deltaY;
    
    transformElement(zid, {x:newX, y:newY});
  }
  
  var transformCache = [];
  function transformElement(zid, obj) {
    
    var cz = MyZoom[zid].resultantZoom;
    if(! transformCache[zid]) transformCache[zid] = {};
    
    if(obj===-1) { // restore
      var matrix = MyZoom[zid].elem.style.transform;
      var a = matrix.match(/^matrix\(([-e+.0-9]+),\s*0,\s*0,\s*\1,\s*(-?[0-9.e+-]+),\s*(-?[0-9.e+-]+)\)$/i);
      if(!a) return log({matrix})
      
      cz.A[0][0] = a[1];
      cz.A[1][1] = a[1];
      cz.b[0] = a[2];
      cz.b[1] = a[3];
    }
    else {
      if(obj.scale) {
        cz.A[0][0] = obj.scale;
        cz.A[1][1] = obj.scale;
        transformCache[zid].scale = obj.scale;
      }
      
      if(obj.hasOwnProperty('x')) {
        cz.b[0] = Math.round(obj.x);
        cz.b[1] = Math.round(obj.y);
        transformCache[zid].x = Math.round(obj.x);
        transformCache[zid].y = Math.round(obj.y);
      }
    }
    
    if(obj.onlysave) return;
    setMyZoom(zid, cz);
  }


  function previewWheel(e) {
    e.preventDefault();
    var zid = parseInt(this.closest('.maxidiv').dataset.zoomid);
    transformElement(zid, -1);
    
    var scale = transformCache[zid].scale;
    if(e.deltaY < 0) var scale2 = scale*1.25;
    else var scale2 = scale*.8;
    
    var ox = e.offsetX*scale, oy = e.offsetY*scale, oh = e.target.offsetHeight*scale, ow = e.target.offsetWidth*scale;
    
    var posy = oy/oh; // 0.9=90%
    
    var nh = oh*scale2/scale;
    var nposy = nh*posy;
    
    var ny = oy*scale2/scale;
    var nx = ox*scale2/scale;
    
    var nby = oy - ny + transformCache[zid].y;
    var nbx = ox - nx + transformCache[zid].x;
    
    transformElement(zid, {x:nbx, y:nby, scale:scale2});
  }

  Zoom.prototype.reset = function() {
    resetMyZoom(this.elem.closest('.maxidiv').dataset.zoomid);
  };
  function resetMyZoom(zid) {
    zid = parseInt(zid);
    var el = MyZoom[zid].elem;
    var div = el.closest('.maxidiv');
    
    var cw = el.offsetWidth, ch = el.offsetHeight;
    var vw = div.offsetWidth, vh = div.offsetHeight;
    
    if(!cw || !vw ) return;
    
    var scale = Math.min(vh/ch, vw/cw);
    transformElement(zid, {
      scale: scale,
      x: (vw-cw*scale)/2,
      y: (vh-ch*scale)/2
    });
  }
  
  function setMyZoom(zid, cz) {
    var css = cz.css;
    var nz = Object.assign({}, cz);
    nz.css = css;
    MyZoom[zid].setZoom(nz);
  }
  

  function initPreviewImg(e){
    makeZoomable(this);
    this.removeEventListener('load', initPreviewImg);
  }
  
  function toggleFS(e){
    var div = this.closest('.maxidiv'), body = this.closest('body');
    var zid = parseInt(div.dataset.zoomid);
    var matrix = MyZoom[zid].elem.style.transform;
    var a = matrix.match(/^matrix\(([.0-9]+),\s*0,\s*0,\s*\1,\s*(-?[0-9.]+),\s*(-?[0-9.]+)\)$/i);
    if(!a) return log({matrix_undetected:matrix});
    
    
    if(this.value == '+') {
      return transformElement(zid, { scale: a[1]*1.25 });
    }
    
    if(this.value == '-') {
      return transformElement(zid, { scale: a[1]*.8 });
    }
    div.classList.toggle('fullscreen');
    body.classList.toggle('fullscreen--maxi2');
    resetMyZoom(zid);
    var b = div.querySelector('input[value="◱"]'); if(b) {
      b.value = '✕';
    }
    else {
      b = div.querySelector('input[value="✕"]'); if(b) b.value = '◱';
    }
  }
  
  var MyZoom = [], CachedZoom = [], mouseCoords = [];
  function makeZoomable(elem) {
    var div = elem.closest('.maxidiv');
    div.classList.add('frame');
    div.insertAdjacentHTML('afterbegin',
        '<div class="maxifullscreen">'
          +'<input type="button" value="◱" title="Toggle full screen" />'
          +'<input type="button" value="+" title="Zoom in; double-tap picture to reset zoom" />'
          +'<input type="button" value="-" title="Zoom out; double-tap picture to reset zoom" />'
        +'</div>');
    
    var btns = div.querySelectorAll('.maxifullscreen input')
    for(var j=0; j<btns.length; j++) {
      btns[j].addEventListener('click', toggleFS);
    }
    
    let i = MyZoom.length;
    var w = elem.offsetWidth+2, h = elem.offsetHeight+2;
    div.style.height = h + "px";
    if(w) div.dataset.ratio = h/w; // assuming w is not zero
    div.querySelector('div.img').style.height = h + "px";
    
    div.dataset.zoomid = i;
    
      
    MyZoom[i] = new Zoom(elem, {
      rotate: false
    });

    var cz = MyZoom[i].resultantZoom;
    transformElement(i, {
      onlysave: true,
      scale: cz.A[0][0],
      x: cz.b[0],
      y: cz.b[1]
    });
    MyZoom[i].petko = false;
    
    div.addEventListener('wheel', previewWheel);
    div.addEventListener('mouseup', previewMouseClick);
    div.addEventListener('mouseout', previewMouseClick);
    div.addEventListener('mousedown', previewMouseClick);
    div.addEventListener('mousemove', previewMouseMove);
    
    setTimeout(function(){resetMyZoom(i);}, 50);
  }
  
  var maxis = document.querySelectorAll('.maxidiv img');
  
  for(var i=0; i<maxis.length; i++) {
    maxis[i].addEventListener('load', initPreviewImg);
  }
  
  if(maxis.length) {
    window.addEventListener('resize', resizeDivs);
  }
  function resizeDivs() {
    var divs = document.querySelectorAll('.maxidiv');
    for(var i=0; i<divs.length; i++) {
      var div = divs[i];
      var w = div.offsetWidth;
      var ratio = parseFloat(div.dataset.ratio);
      if(ratio) {
        var h = (ratio * w)+'px';
        div.style.height = h;
        div.querySelector('div.img').style.height = h;
      }
      resetMyZoom(parseInt(div.dataset.zoomid));
    }
  }
  
});
